from pathlib import Path
from dataclasses import dataclass, asdict
from zona.config import ZonaConfig, find_config
from zona import util
import yaml


@dataclass
class Layout:
    root: Path
    content: Path
    templates: Path | None
    output: Path

    @classmethod
    def from_input(
        cls, root: Path, output: Path | None = None, validate: bool = True
    ) -> "Layout":
        layout = cls(
            root=root.resolve(),
            content=(root / "content").resolve(),
            templates=(root / "templates").resolve(),
            output=(root / "public").resolve() if not output else output,
        )
        if validate:
            if not layout.content.is_dir():
                raise FileNotFoundError(
                    "Missing required content directory!"
                )

        return layout


def discover_layout(
    cli_root: Path | None = None, cli_output: Path | None = None
) -> Layout:
    if cli_root:
        root = cli_root
    else:
        config = find_config(cli_root)
        if config:
            root = config.parent
        else:
            root = Path.cwd()
    return Layout.from_input(root, cli_output)


def initialize_site(root: Path | None = None):
    # initialize a new project
    if not root:
        root = Path.cwd()
    root = root.absolute().resolve()
    config = find_config(root)
    if config is not None:
        raise FileExistsError(f"Config file already exists at {config}")
    # create requires layout
    layout = Layout.from_input(root=root, validate=False)
    # load template resources
    templates = util.get_resources("templates")
    for dir, resources in [
        (layout.root, None),
        (layout.content, None),
        (layout.templates, templates),
    ]:
        if not dir.is_dir():
            dir.mkdir()
        if resources is not None:
            for r in resources:
                Path(r.name).write_text(r.contents)

    config_path = layout.root / "config.yml"
    config = ZonaConfig()
    with open(config_path, "w") as f:
        yaml.dump(
            asdict(config),
            f,
            sort_keys=False,
            default_flow_style=False,
            indent=2,
        )
