from rich import print
from typing import Any, override
from pathlib import Path
from marko.inline import Link, Image
from marko.block import FencedCode
from marko.html_renderer import HTMLRenderer
from marko.parser import Parser
from zona.config import ZonaConfig
from zona.layout import Layout

from pygments import highlight
from pygments.lexers import get_lexer_by_name, TextLexer
from pygments.formatters import HtmlFormatter

from zona import util
from zona.models import Item
from zona.log import get_logger

logger = get_logger()


class ZonaRenderer(HTMLRenderer):
    def __init__(
        self,
        config: ZonaConfig | None,
        resolve: bool = False,
        source: Path | None = None,
        layout: Layout | None = None,
        item_map: dict[Path, Item] | None = None,
    ):
        super().__init__()
        self.resolve: bool = resolve
        if self.resolve:
            assert source is not None
            assert layout is not None
            assert item_map is not None
            self.source: Path = source.resolve()
            self.layout: Layout = layout
            self.item_map: dict[Path, Item] = item_map
            self.config: ZonaConfig | None = config

    @override
    def render_link(self, element: Link):
        href = element.dest
        assert isinstance(href, str)
        if self.resolve:
            cur = Path(href)
            _href = href
            if href.startswith("/"):
                # resolve relative to content root
                resolved = (
                    self.layout.content / cur.relative_to("/")
                ).resolve()
            else:
                # treat as relative link and try to resolve
                resolved = (self.source.parent / cur).resolve()
            # only substitute if link points to an actual file
            if resolved.exists():
                item = self.item_map.get(resolved)
                if item:
                    href = util.normalize_url(item.url)
                    logger.debug(
                        f"Link in file {self.source}: {_href} resolved to {href}"
                    )
                else:
                    logger.debug(
                        f"Warning: resolved path {resolved} not found in item map"
                    )
        body: Any = self.render_children(element)
        return f'<a href="{href}" target="_blank">{body}</a>'

    # TODO: image compression/dithering?
    @override
    def render_image(self, element: Image):
        # get label text from children
        text = self.render_children(element)
        title = element.title or ""
        caption = f"<small>{text}</small>" if text else ""
        return (
            f'<div class="image-container">\n'
            # TODO: convert to plaintext and add as alt attribute
            f'<img src="{element.dest}" title="{title}">\n'
            f"{caption}</div>"
        )

    @override
    def render_fenced_code(self, element: FencedCode):
        assert self.config
        config = self.config.markdown.syntax_highlighting
        code = "".join(child.children for child in element.children)  # type: ignore
        lang = element.lang or "text"
        if not config.enabled:
            return f"<pre><code>{code}</code></pre>"

        try:
            lexer = get_lexer_by_name(lang, stripall=False)
        except Exception:
            lexer = TextLexer(stripall=False)  # type: ignore

        formatter = HtmlFormatter(
            style=config.theme,
            nowrap=not config.wrap,
            noclasses=True,
        )
        highlighted = highlight(code, lexer, formatter)  # type: ignore

        return (
            f'<pre class="code-block language-{lang}">'
            f"<code>{highlighted}</code></pre>"
        )


def md_to_html(
    content: str,
    config: ZonaConfig | None,
    resolve_links: bool = False,
    source: Path | None = None,
    layout: Layout | None = None,
    item_map: dict[Path, Item] | None = None,
) -> str:
    if resolve_links and (
        source is None or layout is None or item_map is None
    ):
        raise TypeError(
            "md_to_html() missing source and ctx when resolve_links is true"
        )
    parser = Parser()
    ast = parser.parse(content)
    renderer = ZonaRenderer(
        config,
        resolve_links,
        source,
        layout=layout,
        item_map=item_map,
    )
    return renderer.render(ast)
