from dataclasses import dataclass
from rich import print
from pathlib import Path
from datetime import date

from dacite.config import Config
from dacite.core import from_dict
from dacite.exceptions import DaciteError
from yaml import YAMLError
import zona.util
import frontmatter


@dataclass
class Metadata:
    title: str
    date: date
    description: str | None
    style: str | None = "/static/style.css"
    header: bool = True
    footer: bool = True
    template: str = "page.html"
    post: bool | None = None


def parse_metadata(path: Path) -> tuple[Metadata, str]:
    """
    Parses a file and returns parsed Metadata and its content. Defaults
    are applied for missing fields. If there is no metadata, a Metadata
    with default values is returned.

    Raises:
        ValueError: If the metadata block is malformed in any way.
    """
    try:
        post = frontmatter.load(str(path))
    except YAMLError as e:
        raise ValueError(f"YAML frontmatter error in {path}: {e}")
    raw_meta = post.metadata or {}
    defaults = {
        "title": zona.util.filename_to_title(path),
        "date": date.fromtimestamp(path.stat().st_mtime),
    }
    meta = {**defaults, **raw_meta}
    try:
        metadata = from_dict(
            data_class=Metadata,
            data=meta,
            config=Config(check_types=True, strict=True),
        )
    except DaciteError as e:
        raise ValueError(f"Malformed metadata in {path}: {e}")
    return metadata, post.content
