from dataclasses import dataclass
from pathlib import Path
from typing import Literal
from jinja2 import Environment, FileSystemLoader, select_autoescape
from jinja2.environment import Template
from zona.models import Item
from zona.markdown import md_to_html
from zona import util


def get_header(root: Path) -> str | None:
    md_header = root / "header.md"
    html_header = root / "header.html"
    if md_header.exists():
        return md_to_html(md_header.read_text(), None)
    elif html_header.exists():
        return html_header.read_text()


def get_footer(root: Path) -> str | None:
    md_footer = root / "footer.md"
    html_footer = root / "footer.html"
    if md_footer.exists():
        return md_to_html(md_footer.read_text(), None)
    elif html_footer.exists():
        return html_footer.read_text()


class TemplateEnv:
    def __init__(self, user_dir: Path | None):
        default_path = util.get_resource_dir("templates")
        self.default: Environment = Environment(
            loader=FileSystemLoader(default_path),
            autoescape=select_autoescape(["html", "xml"]),
        )
        self.user: Environment | None = (
            Environment(
                loader=FileSystemLoader(user_dir),
                autoescape=select_autoescape(["html", "xml"]),
            )
            if user_dir
            else None
        )

    def get_template(self, name: str) -> Template:
        for env in (
            [self.user, self.default] if self.user else [self.default]
        ):
            if name in env.list_templates():
                return env.get_template(name)
        raise FileNotFoundError(f"Template {name} could not be found!")


# TODO: add next/prev post button logic to posts
# TODO: add a recent posts element that can be included elsewhere?
class Templater:
    def __init__(
        self, root: Path, template_dir: Path | None, post_list: list[Item]
    ):
        self.env: TemplateEnv = TemplateEnv(template_dir)
        self.root: Path = root
        self.header: str | None = get_header(self.root)
        self.footer: str | None = get_footer(self.root)
        self.post_list: list[Item] = post_list

    def render_item(self, item: Item, content: str) -> str:
        env = self.env
        meta = item.metadata
        assert meta is not None
        template = env.get_template(
            meta.template
            if meta.template.endswith(".html")
            else meta.template + ".html"
        )
        header: str | Literal[False] = (
            self.header if self.header and meta.header else False
        )
        footer: str | Literal[False] = (
            self.footer if self.footer and meta.footer else False
        )
        return template.render(
            content=content,
            url=item.url,
            metadata=meta,
            header=header,
            footer=footer,
            post_list=self.post_list,
        )
